@@ -2,15 +2,21 @@ require 'forwardable'
 require 'delegate'
 require 'cucumber/errors'
 require 'cucumber/multiline_argument'
+require 'cucumber/formatter/backtrace_filter'
 require 'cucumber/formatter/legacy_api/ast'
 
 module Cucumber
   module Formatter
     module LegacyApi
 
-      Adapter = Struct.new(:formatter, :results, :support_code, :config) do
+      Adapter = Struct.new(:formatter, :results, :config) do
         extend Forwardable
 
+        def initialize(*)
+          super
+          @matches = collect_matches
+        end
+
         def_delegators :formatter,
           :ask
 
@@ -29,13 +35,13 @@ module Cucumber
 
         def after_test_step(test_step, result)
           printer.after_test_step(test_step, result)
-          formatter.after_test_step(test_step, result)
+          formatter.after_test_step(test_step, result.with_filtered_backtrace(Cucumber::Formatter::BacktraceFilter))
         end
 
         def after_test_case(test_case, result)
           record_test_case_result(test_case, result)
           printer.after_test_case(test_case, result)
-          formatter.after_test_case(test_case, result)
+          formatter.after_test_case(test_case, result.with_filtered_backtrace(Cucumber::Formatter::BacktraceFilter))
         end
 
         def puts(*messages)
@@ -50,7 +56,7 @@ module Cucumber
         private
 
         def printer
-          @printer ||= FeaturesPrinter.new(formatter, results, support_code, config).before
+          @printer ||= FeaturesPrinter.new(formatter, results, config, @matches).before
         end
 
         def record_test_case_result(test_case, result)
@@ -58,8 +64,16 @@ module Cucumber
           results.scenario_visited(scenario)
         end
 
+        def collect_matches
+          result = {}
+          config.on_event(:step_match) do |event|
+            result[event.test_step.source.last] = event.step_match
+          end
+          result
+        end
+
         require 'cucumber/core/test/timer'
-        FeaturesPrinter = Struct.new(:formatter, :results, :support_code, :config) do
+        FeaturesPrinter = Struct.new(:formatter, :results, :config, :matches) do
           extend Forwardable
 
           def before
@@ -88,7 +102,7 @@ module Cucumber
           def feature(node, *)
             if node != @current_feature
               @child.after if @child
-              @child = FeaturePrinter.new(formatter, results, support_code, config, node).before
+              @child = FeaturePrinter.new(formatter, results, matches, config, node).before
               @current_feature = node
             end
           end
@@ -124,6 +138,7 @@ module Cucumber
           def timer
             @timer ||= Cucumber::Core::Test::Timer.new
           end
+
         end
 
         module TestCaseSource
@@ -172,9 +187,9 @@ module Cucumber
 
           require 'ostruct'
           class StepSource < OpenStruct
-            def build_step_invocation(indent, support_code, config, messages, embeddings)
+            def build_step_invocation(indent, matches, config, messages, embeddings)
               step_result.step_invocation(
-                step_match(support_code),
+                matches.fetch(step) { NoStepMatch.new(step, step.name) },
                 step,
                 indent,
                 background,
@@ -183,14 +198,6 @@ module Cucumber
                 embeddings
               )
             end
-
-            private
-
-            def step_match(support_code)
-              support_code.step_match(step.name)
-            rescue Cucumber::Undefined
-              NoStepMatch.new(step, step.name)
-            end
           end
 
         end
@@ -202,13 +209,14 @@ module Cucumber
           end
         end
 
-        FeaturePrinter = Struct.new(:formatter, :results, :support_code, :config, :node) do
+        FeaturePrinter = Struct.new(:formatter, :results, :matches, :config, :node) do
 
           def before
             formatter.before_feature(node)
-            Ast::Comments.new(node.comments).accept(formatter)
+            language_comment = node.language.iso_code != 'en' ? ["# language: #{node.language.iso_code}"] : []
+            Ast::Comments.new(language_comment + node.comments).accept(formatter)
             Ast::Tags.new(node.tags).accept(formatter)
-            formatter.feature_name node.keyword, indented(node.legacy_conflated_name_and_description)
+            formatter.feature_name node.keyword, node.legacy_conflated_name_and_description
             @delayed_messages = []
             @delayed_embeddings = []
             self
@@ -249,7 +257,7 @@ module Cucumber
             @delayed_messages = []
             @delayed_embeddings = []
 
-            @child.after_test_case if @child
+            @child.after_test_case(test_case, test_case_result) if @child
             @previous_test_case_background = @current_test_case_background
             @previous_test_case_scenario_outline = current_test_step_source && current_test_step_source.scenario_outline
           end
@@ -271,7 +279,7 @@ module Cucumber
 
           def after_step_hook(hook, result)
             p current_test_step_source if current_test_step_source.step.nil?
-            line = StepBacktraceLine.new(current_test_step_source.step)
+            line = current_test_step_source.step.backtrace_line
             @child.after_step_hook Ast::HookResult.new(LegacyResultBuilder.new(result).
               append_to_exception_backtrace(line), @delayed_messages, @delayed_embeddings)
             @delayed_messages = []
@@ -321,7 +329,7 @@ module Cucumber
               if same_background_as_previous_test_case?(source)
                 HiddenBackgroundPrinter.new(formatter, source.background)
               else
-                BackgroundPrinter.new(formatter, source.background, before_hook_results)
+                BackgroundPrinter.new(formatter, node, source.background, before_hook_results)
               end
             elsif source.scenario
               ScenarioPrinter.new(formatter, source.scenario, before_hook_results)
@@ -355,14 +363,14 @@ module Cucumber
 
             if @failed_hidden_background_step
               indent = Indent.new(@child.node)
-              step_invocation = @failed_hidden_background_step.build_step_invocation(indent, support_code, config, messages = [], embeddings = [])
+              step_invocation = @failed_hidden_background_step.build_step_invocation(indent, matches, config, messages = [], embeddings = [])
               @child.step_invocation(step_invocation, @failed_hidden_background_step)
               @failed_hidden_background_step = nil
             end
 
             unless @last_step == current_test_step_source.step
               indent ||= Indent.new(@child.node)
-              step_invocation = current_test_step_source.build_step_invocation(indent, support_code, config, @delayed_messages, @delayed_embeddings)
+              step_invocation = current_test_step_source.build_step_invocation(indent, matches, config, @delayed_messages, @delayed_embeddings)
               results.step_visited step_invocation
               @child.step_invocation(step_invocation, current_test_step_source)
               @last_step = current_test_step_source.step
@@ -412,15 +420,6 @@ module Cucumber
             to.class.name == ScenarioOutlinePrinter.name
           end
 
-          def indented(nasty_old_conflation_of_name_and_description)
-            indent = ""
-            nasty_old_conflation_of_name_and_description.split("\n").map do |l|
-              s = "#{indent}#{l}"
-              indent = "  "
-              s
-            end.join("\n")
-          end
-
         end
 
         module PrintsAfterHooks
@@ -448,7 +447,7 @@ module Cucumber
           end
         end
 
-        BackgroundPrinter = Struct.new(:formatter, :node, :before_hook_results) do
+        BackgroundPrinter = Struct.new(:formatter, :feature, :node, :before_hook_results) do
 
           def after_test_case(*)
           end
@@ -457,7 +456,8 @@ module Cucumber
           end
 
           def before
-            formatter.before_background node
+            formatter.before_background Ast::Background.new(feature, node)
+            Ast::Comments.new(node.comments).accept(formatter)
             formatter.background_name node.keyword, node.legacy_conflated_name_and_description, node.location.to_s, indent.of(node)
             before_hook_results.accept(formatter)
             self
@@ -477,7 +477,7 @@ module Cucumber
 
           def after
             @child.after if @child
-            formatter.after_background(node)
+            formatter.after_background(Ast::Background.new(feature, node))
             self
           end
 
@@ -519,6 +519,7 @@ module Cucumber
 
           def before
             formatter.before_feature_element(node)
+            Ast::Comments.new(node.comments).accept(formatter)
             Ast::Tags.new(node.tags).accept(formatter)
             formatter.scenario_name node.keyword, node.legacy_conflated_name_and_description, node.location.to_s, indent.of(node)
             before_hook_results.accept(formatter)
@@ -528,23 +529,21 @@ module Cucumber
           def step_invocation(step_invocation, source)
             @child ||= StepsPrinter.new(formatter).before
             @child.step_invocation step_invocation
-            @last_step_result = source.step_result
           end
 
           def after_step_hook(result)
             result.accept formatter
           end
 
-          def after_test_case(*args)
+          def after_test_case(test_case, result)
+            @test_case_result = result
             after
           end
 
           def after
             return if @done
             @child.after if @child
-            # TODO - the last step result might not accurately reflect the
-            # overall scenario result.
-            scenario = last_step_result.scenario(node.name, node.location)
+            scenario = LegacyResultBuilder.new(@test_case_result).scenario(node.name, node.location)
             after_hook_results.accept(formatter)
             formatter.after_feature_element(scenario)
             @done = true
@@ -553,10 +552,6 @@ module Cucumber
 
           private
 
-          def last_step_result
-            @last_step_result || LegacyResultBuilder.new(Core::Test::Result::Unknown.new)
-          end
-
           def indent
             @indent ||= Indent.new(node)
           end
@@ -593,6 +588,7 @@ module Cucumber
 
           def before
             formatter.before_feature_element(node)
+            Ast::Comments.new(node.comments).accept(formatter)
             Ast::Tags.new(node.tags).accept(formatter)
             formatter.scenario_name node.keyword, node.legacy_conflated_name_and_description, node.location.to_s, indent.of(node)
             OutlineStepsPrinter.new(formatter, configuration, indent).print(node)
@@ -614,8 +610,8 @@ module Cucumber
             @child.examples_table_row(node, before_hook_results)
           end
 
-          def after_test_case
-            @child.after_test_case
+          def after_test_case(test_case, result)
+            @child.after_test_case(test_case, result)
           end
 
           def after
@@ -693,6 +689,8 @@ module Cucumber
 
           def before
             formatter.before_examples(node)
+            Ast::Comments.new(node.comments).accept(formatter)
+            Ast::Tags.new(node.tags).accept(formatter)
             formatter.examples_name(node.keyword, node.legacy_conflated_name_and_description)
             formatter.before_outline_table(legacy_table)
             if !configuration.expand?
@@ -714,7 +712,7 @@ module Cucumber
           end
 
           def after_test_case(*args)
-            @child.after_test_case
+            @child.after_test_case(*args)
           end
 
           def after
@@ -743,9 +741,9 @@ module Cucumber
               max_width.result
             end
 
-            require 'gherkin/formatter/escaping'
+            require 'cucumber/gherkin/formatter/escaping'
             FindMaxWidth = Struct.new(:index) do
-              include ::Gherkin::Formatter::Escaping
+              include ::Cucumber::Gherkin::Formatter::Escaping
 
               def examples_table(table, &descend)
                 @result = char_length_of(table.header.values[index])
@@ -803,6 +801,7 @@ module Cucumber
           end
 
           def before
+            Ast::Comments.new(node.comments).accept(formatter)
             formatter.before_table_row(node)
             self
           end
@@ -822,6 +821,7 @@ module Cucumber
         class TableRowPrinter < TableRowPrinterBase
           def before
             before_hook_results.accept(formatter)
+            Ast::Comments.new(node.comments).accept(formatter)
             formatter.before_table_row(node)
             self
           end
@@ -995,66 +995,22 @@ module Cucumber
             return filtered_step_exception(step) if @exception
             return nil unless @status == :undefined && configuration.strict?
             @exception = Cucumber::Undefined.from(@result, step.name)
+            @exception.backtrace << step.backtrace_line
             filtered_step_exception(step)
           end
 
           def filtered_exception
-            BacktraceFilter.new(@exception.dup).exception
+            Cucumber::Formatter::BacktraceFilter.new(@exception.dup).exception
           end
 
           def filtered_step_exception(step)
             exception = filtered_exception
-            exception.backtrace << StepBacktraceLine.new(step).to_s
-            return BacktraceFilter.new(exception).exception
+            return Cucumber::Formatter::BacktraceFilter.new(exception).exception
           end
         end
 
       end
 
-      class StepBacktraceLine < Struct.new(:step)
-        def initialize(step)
-          raise ArgumentError if step.nil?
-          super
-        end
-
-        def to_s
-          step.backtrace_line
-        end
-      end
-
-      require 'cucumber/platform'
-      class BacktraceFilter
-        BACKTRACE_FILTER_PATTERNS = \
-          [/vendor\/rails|lib\/cucumber|bin\/cucumber:|lib\/rspec|gems\/|minitest|test\/unit|.gem\/ruby|lib\/ruby/]
-        if(::Cucumber::JRUBY)
-          BACKTRACE_FILTER_PATTERNS << /org\/jruby/
-        end
-        PWD_PATTERN = /#{::Regexp.escape(::Dir.pwd)}\//m
-
-        def initialize(exception)
-          @exception = exception
-        end
-
-        def exception
-          return @exception if ::Cucumber.use_full_backtrace
-          @exception.backtrace.each{|line| line.gsub!(PWD_PATTERN, "./")}
-
-          filtered = (@exception.backtrace || []).reject do |line|
-            BACKTRACE_FILTER_PATTERNS.detect { |p| line =~ p }
-          end
-
-          if ::ENV['CUCUMBER_TRUNCATE_OUTPUT']
-            # Strip off file locations
-            filtered = filtered.map do |line|
-              line =~ /(.*):in `/ ? $1 : line
-            end
-          end
-
-          @exception.set_backtrace(filtered)
-          @exception
-        end
-      end
-
     end
   end
 end
