@@ -1,3 +1,5 @@
+require 'cucumber/gherkin/i18n'
+
 module Cucumber
   module Formatter
     module LegacyApi
@@ -9,11 +11,10 @@ module Cucumber
         end
 
         def snippet_text(step_keyword, step_name, multiline_arg) #:nodoc:
-          support_code.snippet_text(::Gherkin::I18n.code_keyword_for(step_keyword), step_name, multiline_arg)
-        end
-
-        def unknown_programming_language?
-          support_code.unknown_programming_language?
+          keyword = Cucumber::Gherkin::I18n.code_keyword_for(step_keyword).strip
+          configuration.snippet_generators.map { |generator|
+            generator.call(keyword, step_name, multiline_arg, configuration.snippet_type)
+          }.join("\n")
         end
 
         def scenarios(status = nil)
