@@ -6,12 +6,13 @@ module Cucumber
       include Formatter::Io
 
       def initialize(runtime, path_or_io, options)
-        @io = ensure_io(path_or_io, "rerun")
+        @io = ensure_io(path_or_io)
         @failures = {}
+        @options = options
       end
 
       def after_test_case(test_case, result)
-        return if result.passed?
+        return if result.ok?(@options[:strict])
         @failures[test_case.location.file] ||= []
         @failures[test_case.location.file] << test_case.location.line
       end
