@@ -12,41 +12,39 @@ module Cucumber
 
       def initialize(runtime, path_or_io, options)
         @runtime = runtime
-        @io = ensure_io(path_or_io, "usage")
+        @io = ensure_io(path_or_io)
         @options = options
-        @stepdef_to_match = Hash.new{|h,stepdef_key| h[stepdef_key] = []}
+        @stepdef_to_match = Hash.new { |h, stepdef_key| h[stepdef_key] = [] }
+        @total_duration = 0
+        @matches = {}
+        runtime.configuration.on_event :step_match do |event|
+          @matches[event.test_step.source] = event.step_match
+        end
       end
 
-      def before_features(features)
-        print_profile_information
-      end
+      def after_test_step(test_step, result)
+        return if HookQueryVisitor.new(test_step).hook?
 
-      def before_step(step)
-        @step = step
-        @start_time = Time.now
-      end
-
-      def before_step_result(*args)
-        @duration = Time.now - @start_time
-      end
-
-      def after_step_result(keyword, step_match, multiline_arg, status, exception, source_indent, background, file_colon_line)
+        step_match = @matches[test_step.source]
         step_definition = step_match.step_definition
-        unless step_definition.nil? # nil if it's from a scenario outline
-          stepdef_key = StepDefKey.new(step_definition.regexp_source, step_definition.file_colon_line)
+        stepdef_key = StepDefKey.new(step_definition.regexp_source, step_definition.location)
+        unless @stepdef_to_match[stepdef_key].map { |key| key[:location] }.include? test_step.location
+          duration = DurationExtractor.new(result).result_duration
 
           @stepdef_to_match[stepdef_key] << {
-            :keyword => keyword,
-            :step_match => step_match,
-            :status => status,
-            :file_colon_line => @step.file_colon_line,
-            :duration => @duration
+            keyword: test_step.source.last.keyword,
+            step_match: step_match,
+            status: result.to_sym,
+            location: test_step.location,
+            duration: duration
           }
         end
         super
       end
 
-      def print_summary(features)
+      private
+
+      def print_summary
         add_unused_stepdefs
         aggregate_info
 
@@ -74,7 +72,7 @@ module Cucumber
         @io.print format_string(stepdef_key.regexp_source, stepdef_key.status)
         if @options[:source]
           indent = max_length - stepdef_key.regexp_source.unpack('U*').length
-          line_comment = "   # #{stepdef_key.file_colon_line}".indent(indent)
+          line_comment = "   # #{stepdef_key.location}".indent(indent)
           @io.print(format_string(line_comment, :comment))
         end
         @io.puts
@@ -87,7 +85,7 @@ module Cucumber
           @io.print format_step(step[:keyword], step[:step_match], step[:status], nil)
           if @options[:source]
             indent = max_length - (step[:keyword].unpack('U*').length + step[:step_match].format_args.unpack('U*').length)
-            line_comment = " # #{step[:file_colon_line]}".indent(indent)
+            line_comment = " # #{step[:location]}".indent(indent)
             @io.print(format_string(line_comment, :comment))
           end
           @io.puts
@@ -129,7 +127,7 @@ module Cucumber
 
       def add_unused_stepdefs
         @runtime.unmatched_step_definitions.each do |step_definition|
-          stepdef_key = StepDefKey.new(step_definition.regexp_source, step_definition.file_colon_line)
+          stepdef_key = StepDefKey.new(step_definition.regexp_source, step_definition.location)
           @stepdef_to_match[stepdef_key] = []
         end
       end
