@@ -5,7 +5,10 @@ require 'cucumber/rb_support/rb_step_definition'
 require 'cucumber/rb_support/rb_hook'
 require 'cucumber/rb_support/rb_transform'
 require 'cucumber/rb_support/snippet'
+require 'cucumber/gherkin/i18n'
 require 'multi_test'
+require 'cucumber/step_match'
+require 'cucumber/step_definition_light'
 
 module Cucumber
   module RbSupport
@@ -21,8 +24,8 @@ module Cucumber
       def initialize(first_proc, second_proc)
         message = "You can only pass a proc to #World once, but it's happening\n"
         message << "in 2 places:\n\n"
-        message << first_proc.backtrace_line('World') << "\n"
-        message << second_proc.backtrace_line('World') << "\n\n"
+        message << RbSupport.backtrace_line(first_proc, 'World') << "\n"
+        message << RbSupport.backtrace_line(second_proc, 'World') << "\n\n"
         message << "Use Ruby modules instead to extend your worlds. See the Cucumber::RbSupport::RbDsl#World RDoc\n"
         message << "or http://wiki.github.com/cucumber/cucumber/a-whole-new-world.\n\n"
         super(message)
@@ -31,34 +34,32 @@ module Cucumber
 
     # The Ruby implementation of the programming language API.
     class RbLanguage
-      include LanguageSupport::LanguageMethods
       attr_reader :current_world,
                   :step_definitions
 
-      Gherkin::I18n.code_keywords.each do |adverb|
-        RbDsl.alias_adverb(adverb)
+      all_keywords = ::Gherkin::DIALECTS.keys.map do |dialect_name|
+        dialect = ::Gherkin::Dialect.for(dialect_name)
+        dialect.given_keywords + dialect.when_keywords + dialect.then_keywords + dialect.and_keywords + dialect.but_keywords
+      end
+      Cucumber::Gherkin::I18n.code_keywords_for(all_keywords.flatten.uniq.sort).each do |adverb|
+        RbDsl.alias_adverb(adverb.strip)
       end
 
-      def initialize(runtime)
+      def initialize(runtime, configuration)
         @runtime = runtime
         @step_definitions = []
         RbDsl.rb_language = self
         @world_proc = @world_modules = nil
+        configuration.register_snippet_generator(Snippet::Generator.new)
       end
 
-      def step_matches(name_to_match, name_to_format)
-        @step_definitions.map do |step_definition|
-          if(arguments = step_definition.arguments_from(name_to_match))
-            StepMatch.new(step_definition, name_to_match, name_to_format, arguments)
-          else
-            nil
+      def step_matches(name_to_match)
+        @step_definitions.reduce([]) { |result, step_definition|
+          if (arguments = step_definition.arguments_from(name_to_match))
+            result << StepMatch.new(step_definition, name_to_match, arguments)
           end
-        end.compact
-      end
-
-      def snippet_text(code_keyword, step_name, multiline_arg, snippet_type = :regexp)
-        snippet_class = typed_snippet_class(snippet_type)
-        snippet_class.new(code_keyword, step_name, multiline_arg).to_s
+          result
+        }
       end
 
       def begin_rb_scenario(scenario)
@@ -91,6 +92,7 @@ module Cucumber
       end
 
       def load_code_file(code_file)
+        return unless File.extname(code_file) == ".rb"
         load File.expand_path(code_file) # This will cause self.add_step_definition, self.add_hook, and self.add_transform to be called from RbDsl
       end
 
@@ -102,8 +104,67 @@ module Cucumber
         @current_world = nil
       end
 
+      def after_configuration(configuration)
+        hooks[:after_configuration].each do |hook|
+          hook.invoke('AfterConfiguration', configuration)
+        end
+      end
+
+      def execute_transforms(args)
+        args.map do |arg|
+          matching_transform = transforms.detect {|transform| transform.match(arg) }
+          matching_transform ? matching_transform.invoke(arg) : arg
+        end
+      end
+
+      def add_hook(phase, hook)
+        hooks[phase.to_sym] << hook
+        hook
+      end
+
+      def clear_hooks
+        @hooks = nil
+      end
+
+      def add_transform(transform)
+        transforms.unshift transform
+        transform
+      end
+
+      def hooks_for(phase, scenario) #:nodoc:
+        hooks[phase.to_sym].select{|hook| scenario.accept_hook?(hook)}
+      end
+
+      def unmatched_step_definitions
+        available_step_definition_hash.keys - invoked_step_definition_hash.keys
+      end
+
+      def available_step_definition(regexp_source, file_colon_line)
+        available_step_definition_hash[StepDefinitionLight.new(regexp_source, file_colon_line)] = nil
+      end
+
+      def invoked_step_definition(regexp_source, file_colon_line)
+        invoked_step_definition_hash[StepDefinitionLight.new(regexp_source, file_colon_line)] = nil
+      end
+
       private
 
+      def available_step_definition_hash
+        @available_step_definition_hash ||= {}
+      end
+
+      def invoked_step_definition_hash
+        @invoked_step_definition_hash ||= {}
+      end
+
+      def hooks
+        @hooks ||= Hash.new{|h,k| h[k] = []}
+      end
+
+      def transforms
+        @transforms ||= []
+      end
+
       def create_world
         if(@world_proc)
           @current_world = @world_proc.call
@@ -132,7 +193,7 @@ module Cucumber
             raise NilWorld.new
           rescue NilWorld => e
             e.backtrace.clear
-            e.backtrace.push(proc.backtrace_line("World"))
+            e.backtrace.push(RbSupport.backtrace_line(proc, "World"))
             raise e
           end
         else
@@ -140,21 +201,16 @@ module Cucumber
         end
       end
 
-      SNIPPET_TYPES = {
-        :regexp => Snippet::Regexp,
-        :classic => Snippet::Classic,
-        :percent => Snippet::Percent
-      }
-
-      def typed_snippet_class(type)
-        SNIPPET_TYPES.fetch(type || :regexp)
-      end
-
       def self.cli_snippet_type_options
-        SNIPPET_TYPES.keys.sort_by(&:to_s).map do |type|
-          SNIPPET_TYPES[type].cli_option_string(type)
+        Snippet::SNIPPET_TYPES.keys.sort_by(&:to_s).map do |type|
+          Snippet::SNIPPET_TYPES[type].cli_option_string(type)
         end
       end
     end
+
+    def self.backtrace_line(proc, name)
+      location = Cucumber::Core::Ast::Location.from_source_location(*proc.source_location)
+      "#{location.to_s}:in `#{name}'"
+    end
   end
 end
