@@ -1,8 +1,8 @@
 module Cucumber
   class Runtime
     class StepHooks
-      def initialize(after)
-        @after = after
+      def initialize(hooks)
+        @hooks = hooks
       end
 
       def apply(test_steps)
@@ -13,8 +13,9 @@ module Cucumber
 
       private
       def after_step_hooks(test_step)
-        @after.map do |action_block|
-          Hooks.after_step_hook(test_step.source, &action_block)
+        @hooks.map do |hook|
+          action = ->(*args) { hook.invoke('AfterStep', [args, test_step]) }
+          Hooks.after_step_hook(test_step.source, hook.location, &action)
         end
       end
     end
