@@ -1,28 +1,22 @@
 require 'cucumber/multiline_argument'
 
 module Cucumber
+
+  # Represents the match found between a Test Step and its activation
   class StepMatch #:nodoc:
     attr_reader :step_definition, :step_arguments
 
-    # Creates a new StepMatch. The +name_to_report+ argument is what's
-    # reported, unless it's is, in which case +name_to_report+ is used instead.
-    #
-    def initialize(step_definition, name_to_match, name_to_report, step_arguments)
-      raise "name_to_match can't be nil" if name_to_match.nil?
+    def initialize(step_definition, step_name, step_arguments)
       raise "step_arguments can't be nil (but it can be an empty array)" if step_arguments.nil?
-      @step_definition, @name_to_match, @name_to_report, @step_arguments = step_definition, name_to_match, name_to_report, step_arguments
+      @step_definition, @name_to_match, @step_arguments = step_definition, step_name, step_arguments
     end
 
     def args
       @step_arguments.map{|g| g.val }
     end
 
-    def name
-      @name_to_report
-    end
-
     def activate(test_step)
-      test_step.with_action do
+      test_step.with_action(@step_definition.location) do
         invoke(MultilineArgument.from_core(test_step.source.last.multiline_arg))
       end
     end
@@ -49,11 +43,15 @@ module Cucumber
     #   lambda { |param| "[#{param}]" }
     #
     def format_args(format = lambda{|a| a}, &proc)
-      @name_to_report || replace_arguments(@name_to_match, @step_arguments, format, &proc)
+      replace_arguments(@name_to_match, @step_arguments, format, &proc)
+    end
+
+    def location
+      @step_definition.location
     end
 
     def file_colon_line
-      @step_definition.file_colon_line
+      location.to_s
     end
 
     def backtrace_line
@@ -86,7 +84,7 @@ module Cucumber
     end
 
     def inspect #:nodoc:
-      sprintf("#<%s:0x%x>", self.class, self.object_id)
+      "#<#{self.class}: #{location}>"
     end
 
     private
@@ -113,6 +111,11 @@ module Cucumber
       @name
     end
 
+    def location
+      raise "No location for #{@step}" unless @step.location
+      @step.location
+    end
+
     def file_colon_line
       raise "No file:line for #{@step}" unless @step.file_colon_line
       @step.file_colon_line
