@@ -1,6 +1,5 @@
 require 'spec_helper'
 require 'yaml'
-require 'gherkin/formatter/model'
 
 module Cucumber
   module Cli
@@ -51,7 +50,7 @@ module Cucumber
             Cucumber.wants_to_quit = false
           end
 
-          it "registers as a failure" do
+          it "exits with error code" do
             results = double('results', :failure? => false)
 
             allow_any_instance_of(Runtime).to receive(:run!)
@@ -59,41 +58,20 @@ module Cucumber
 
             Cucumber.wants_to_quit = true
 
-            expect(kernel).to receive(:exit).with(1)
+            expect(kernel).to receive(:exit).with(2)
 
             subject.execute!
           end
         end
       end
 
-      describe "--format with class" do
-        describe "in module" do
-          let(:double_module) { double('module') }
-          let(:formatter) { double('formatter') }
-
-          it "resolves each module until it gets Formatter class" do
-            cli = Main.new(%w{--format ZooModule::MonkeyFormatterClass}, stdin, stdout, stderr, kernel)
-
-            allow(Object).to receive(:const_defined?) { true }
-            allow(double_module).to receive(:const_defined?) { true }
-
-            expect(Object).to receive(:const_get).with('ZooModule', false) { double_module }
-            expect(double_module).to receive(:const_get).with('MonkeyFormatterClass', false) { double('formatter class', :new => formatter) }
-
-            expect(kernel).to receive(:exit).with(0)
-
-            cli.execute!
-          end
-        end
-      end
-
       [ProfilesNotDefinedError, YmlLoadError, ProfileNotFound].each do |exception_klass|
         it "rescues #{exception_klass}, prints the message to the error stream" do
           configuration = double('configuration')
 
           allow(Configuration).to receive(:new) { configuration }
           allow(configuration).to receive(:parse!).and_raise(exception_klass.new("error message"))
-          allow(kernel).to receive(:exit).with(1)
+          allow(kernel).to receive(:exit).with(2)
 
           subject.execute!
 
