@@ -40,7 +40,7 @@ module Cucumber
 
         describe "with a step that embeds a snapshot" do
           define_steps do
-            Given(/snap/) { 
+            Given(/snap/) {
               RSpec::Mocks.allow_message(File, :file?) { true }
               embed('out/snapshot.jpeg', 'image/jpeg')
             }
@@ -87,7 +87,25 @@ module Cucumber
 
           it { expect(@out.string).to match(/^\<!DOCTYPE/) }
           it { expect(@out.string).to match(/\<\/html\>$/) }
-          it { expect(@doc).to have_css_node('.feature .comment', /Healthy/) }
+
+          it "nests the comment within the feature" do
+            expect(@doc).to have_css_node('.feature .comment', /Healthy/)
+          end
+
+          it "properly closes the comment" do
+            expect(@out.string).to match(%r{<pre class="comment"># Healthy<br/></pre>})
+          end
+        end
+
+        describe "with a comment at scenario level" do
+          define_feature <<-FEATURE
+            Feature: Foo
+              # Healthy Scenario
+              Scenario:
+                Given passing
+          FEATURE
+
+          it { expect(@doc).to have_css_node('.scenario .comment', /Healthy Scenario/) }
         end
 
         describe "with a tag" do
@@ -273,9 +291,49 @@ module Cucumber
           it { expect(@doc).not_to have_css_node('.feature .scenario .backtrace', //) }
         end
 
+        context "with a before hook that fails" do
+          describe "in a scenario" do
+            define_steps do
+              Before { raise 'eek' }
+              Given(/yay/) {}
+            end
+
+            define_feature(<<-FEATURE)
+            Feature: shouting
+              Scenario:
+                Given yay
+              FEATURE
+
+            it { expect(@out.string).to include('makeRed(\'scenario_1\')') }
+            it { expect(@doc).not_to have_css_node('.feature .scenario .step.failed', //) }
+            it { expect(@doc).to have_css_node('.feature .scenario .step.skipped', /yay/) }
+          end
+
+          describe "in a scenario outline" do
+            define_steps do
+              Before { raise 'eek' }
+              Given(/yay/) {}
+            end
+
+            define_feature(<<-FEATURE)
+            Feature: shouting
+              Scenario Outline:
+                Given <step>
+                Examples:
+                | step |
+                | yay  |
+              FEATURE
+
+            it { expect(@out.string).to include('makeRed(\'scenario_1\')') }
+            it { expect(@doc).not_to have_css_node('.feature .scenario .step.failed', //) }
+            it { expect(@doc).to have_css_node('.feature .scenario .step.skipped', /yay/) }
+          end
+
+        end
+
         describe "with a step that embeds a snapshot" do
           define_steps do
-            Given(/snap/) { 
+            Given(/snap/) {
               RSpec::Mocks.allow_message(File, :file?) { true }
               embed('snapshot.jpeg', 'image/jpeg')
             }
@@ -289,7 +347,7 @@ module Cucumber
 
           it { expect(@doc.css('.embed img').first.attributes['src'].to_s).to eq "snapshot.jpeg" }
         end
-        
+
         describe "with a step that embeds a text" do
           define_steps do
             Given(/log/) { embed('log.txt', 'text/plain') }
@@ -428,10 +486,10 @@ module Cucumber
         end
       end
       context "in --expand mode" do
-        let(:runtime)   { Runtime.new({:expand => true})}
+        let(:options)   { { expand: true } }
         before(:each) do
           @out = StringIO.new
-          @formatter = Html.new(runtime, @out, {})
+          @formatter = Html.new(runtime, @out, {:expand => true})
           run_defined_feature
           @doc = Nokogiri.HTML(@out.string)
         end
@@ -453,6 +511,27 @@ module Cucumber
 
           it { expect(@doc.css('pre').map { |pre| /^(Given|Then|When)/.match(pre.text)[1] }).to eq ["Given", "Given"] }
         end
+
+        describe "with a scenario outline and a before hook that fails" do
+          define_steps do
+            Before { raise 'eek' }
+            Given(/yay/) {}
+          end
+
+          define_feature(<<-FEATURE)
+          Feature: shouting
+            Scenario Outline:
+              Given <step>
+              Examples:
+              | step |
+              | yay  |
+            FEATURE
+
+          it { expect(@out.string).to include('makeRed(\'scenario_1\')') }
+          it { expect(@out.string).to include('makeRed(\'scenario_1_1\')') }
+          it { expect(@doc).not_to have_css_node('.feature .scenario .step.failed', //) }
+          it { expect(@doc).to have_css_node('.feature .scenario .step.skipped', /yay/) }
+        end
       end
     end
   end
