@@ -22,7 +22,7 @@ module Cucumber
       context "With no options" do
         before(:each) do
           allow(File).to receive(:directory?) { true }
-          @formatter = TestDoubleJunitFormatter.new(runtime, '', {})
+          @formatter = TestDoubleJunitFormatter.new(actual_runtime.configuration.with_options(out_stream: ''))
         end
 
         after(:each) do
@@ -59,7 +59,7 @@ module Cucumber
             end
           end
 
-          it { expect(@doc.xpath('//testsuite/system-out').first.content).to match(/\s+boo boo\s+/) }
+          it { expect(@doc.xpath('//testsuite/testcase/system-out').first.content).to match(/\s+boo boo\s+/) }
         end
 
         describe "a feature with no name" do
@@ -92,12 +92,12 @@ module Cucumber
 
             it { expect(@doc.to_s).to match /One passing scenario, one failing scenario/ }
 
-            it 'has a root system-out node' do
-              expect(@doc.xpath('//testsuite/system-out').size).to eq 1
+            it 'has not a root system-out node' do
+              expect(@doc.xpath('//testsuite/system-out').size).to eq 0
             end
 
-            it 'has a root system-err node' do
-              expect(@doc.xpath('//testsuite/system-err').size).to eq 1
+            it 'has not a root system-err node' do
+              expect(@doc.xpath('//testsuite/system-err').size).to eq 0
             end
 
             it 'has a system-out node under <testcase/>' do
@@ -200,7 +200,7 @@ module Cucumber
         let(:runtime)   { Runtime.new({:expand => true}) }
         before(:each) do
           allow(File).to receive(:directory?) { true }
-          @formatter = TestDoubleJunitFormatter.new(runtime, '', {:expand => true})
+          @formatter = TestDoubleJunitFormatter.new(actual_runtime.configuration.with_options(out_stream: '', :expand => true))
         end
 
         after(:each) do
