@@ -5,6 +5,7 @@ require 'cucumber/runtime/step_hooks'
 require 'cucumber/runtime/before_hooks'
 require 'cucumber/runtime/after_hooks'
 require 'cucumber/filters'
+require 'spec_helper'
 
 module Cucumber
   module Formatter::LegacyApi
@@ -12,9 +13,10 @@ module Cucumber
       include Core::Gherkin::Writer
       include Core
 
-      let(:report)    { Adapter.new(formatter, runtime.results, runtime.support_code, runtime.configuration) }
+      let(:report)    { Adapter.new(formatter, runtime.results, runtime.configuration) }
       let(:formatter) { double('formatter').as_null_object }
       let(:runtime)   { Runtime.new }
+      let(:step_match_search) { SimpleStepDefinitionSearch.new }
 
       Failure = Class.new(StandardError)
 
@@ -28,23 +30,41 @@ module Cucumber
         end
       end
 
-      class SimpleStepDefinitions
-        def find_match(test_step)
-          if test_step.name =~ /pass/
-            return SimpleStepMatch.new {}
+      class SimpleStepDefinitionSearch
+        def call(name)
+          if name =~ /pass/
+            return [SimpleStepMatch.new {}]
           end
 
-          if test_step.name =~ /fail/
-            return SimpleStepMatch.new { raise Failure }
+          if name =~ /fail/
+            return [SimpleStepMatch.new { raise Failure }]
           end
 
-          NoStepMatch.new test_step.source.last, test_step.name
+          []
+        end
+      end
+
+      class HookWrapper
+        def initialize(proc)
+          @proc = proc
+        end
+
+        def source_location
+          @proc.source_location
+        end
+
+        def location
+          source_location
+        end
+
+        def invoke(pseudo_method, arguments, &block)
+          @proc.call
         end
       end
 
       class AddBeforeAndAfterHooks < Core::Filter.new
         def test_case(test_case)
-          steps = before_hooks(test_case.source) + 
+          steps = before_hooks(test_case.source) +
             test_case.test_steps +
             after_hooks(test_case.source)
           test_case.with_steps(steps).describe_to receiver
@@ -55,12 +75,14 @@ module Cucumber
         def before_hooks(source)
           # The adapter is built on the assumption that each test case will have at least one step. This is annoying
           # for tests, but a safe assumption for production use as we always add one hook to initialize the world.
-          [ Hooks.before_hook(source) {} ]
+          hook = proc {}
+          [ Hooks.before_hook(source, hook.source_location, &hook) ]
         end
 
         def after_hooks(source)
           # We add an after hook to make sure the adapter can cope with it
-          [ Hooks.after_hook(source) {} ]
+          hook = proc {}
+          [ Hooks.after_hook(source, hook.source_location, &hook) ]
         end
       end
 
@@ -454,6 +476,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -522,6 +546,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -606,6 +632,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -635,6 +663,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -701,6 +731,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -775,6 +807,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -790,6 +824,8 @@ module Cucumber
                         :after_outline_table,
                       :after_examples,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -936,6 +972,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1003,6 +1041,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1058,6 +1098,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1123,6 +1165,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1184,6 +1228,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1199,6 +1245,8 @@ module Cucumber
                         :after_outline_table,
                       :after_examples,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1258,6 +1306,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1287,6 +1337,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1339,6 +1391,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1456,6 +1510,8 @@ module Cucumber
                       :after_steps,
                       :before_examples_array,
                         :before_examples,
+                          :before_tags,
+                          :after_tags,
                           :examples_name,
                           :before_outline_table,
                             :scenario_name,
@@ -1484,13 +1540,14 @@ module Cucumber
 
           class FailingAfterStepHook
             def find_after_step_hooks(test_case)
-              Runtime::StepHooks.new [-> { raise Failure }]
+              failing_hook = HookWrapper.new(proc { raise Failure })
+              Runtime::StepHooks.new [failing_hook]
             end
           end
 
           it 'prints the exception within the step' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAfterStepHooks.new(FailingAfterStepHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1529,13 +1586,14 @@ module Cucumber
         context 'with exception in a single before hook' do
           class FailingBeforeHook
             def apply_before_hooks(test_case)
-              Runtime::BeforeHooks.new([proc { raise Failure }]).apply_to(test_case)
+              failing_hook = HookWrapper.new(proc { raise Failure })
+              Runtime::BeforeHooks.new([failing_hook], RunningTestCase.new(test_case)).apply_to(test_case)
             end
           end
 
           it 'prints the exception after the scenario name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyBeforeHooks.new(FailingBeforeHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1573,7 +1631,7 @@ module Cucumber
 
           it 'prints the exception after the background name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyBeforeHooks.new(FailingBeforeHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1625,7 +1683,7 @@ module Cucumber
 
           it 'prints the exception before the examples table row' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyBeforeHooks.new(FailingBeforeHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1660,6 +1718,8 @@ module Cucumber
                       :after_steps,
                       :before_examples_array,
                         :before_examples,
+                          :before_tags,
+                          :after_tags,
                           :examples_name,
                           :before_outline_table,
                             :before_table_row,
@@ -1688,13 +1748,15 @@ module Cucumber
           # the result of the first one.
           class FailingAndPassingBeforeHooks
             def apply_before_hooks(test_case)
-              Runtime::BeforeHooks.new([proc { raise Failure }, proc { }]).apply_to(test_case)
+              failing_hook = HookWrapper.new(proc { raise Failure })
+              passing_hook = HookWrapper.new(proc {})
+              Runtime::BeforeHooks.new([failing_hook, passing_hook], RunningTestCase.new(test_case)).apply_to(test_case)
             end
           end
 
           it 'prints the exception after the scenario name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyBeforeHooks.new(FailingAndPassingBeforeHooks.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1735,13 +1797,14 @@ module Cucumber
 
           class FailingAfterHook
             def apply_after_hooks(test_case)
-              Runtime::AfterHooks.new([proc { raise Failure }]).apply_to(test_case)
+              failing_hook = HookWrapper.new(proc { raise Failure })
+              Runtime::AfterHooks.new([failing_hook], RunningTestCase.new(test_case)).apply_to(test_case)
             end
           end
 
           it 'prints the exception after the steps' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAfterHooks.new(FailingAfterHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1779,7 +1842,7 @@ module Cucumber
 
           it 'prints the exception after the examples table row' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAfterHooks.new(FailingAfterHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1814,6 +1877,8 @@ module Cucumber
                     :after_steps,
                     :before_examples_array,
                       :before_examples,
+                        :before_tags,
+                        :after_tags,
                         :examples_name,
                         :before_outline_table,
                           :before_table_row,
@@ -1840,13 +1905,15 @@ module Cucumber
         context 'with exception in the first of several after hooks' do
           class FailingThenPassingAfterHooks
             def apply_after_hooks(test_case)
-              Runtime::AfterHooks.new([proc { raise Failure }, proc {}]).apply_to(test_case)
+              failing_hook = HookWrapper.new(proc { raise Failure })
+              passing_hook = HookWrapper.new(proc {})
+              Runtime::AfterHooks.new([failing_hook, passing_hook], RunningTestCase.new(test_case)).apply_to(test_case)
             end
           end
 
           it 'prints the exception after the steps' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAfterHooks.new(FailingThenPassingAfterHooks.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1886,7 +1953,7 @@ module Cucumber
         context 'with an exception in an after hook but no steps' do
           it 'prints the exception after the scenario name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAfterHooks.new(FailingAfterHook.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1926,7 +1993,7 @@ module Cucumber
 
           it 'prints the exception after the scenario name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAroundHooks.new(FailingAroundHookBeforeRunningTestCase.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -1966,7 +2033,7 @@ module Cucumber
 
           it 'prints the exception after the scenario name' do
             filters = [
-              Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+              Filters::ActivateSteps.new(step_match_search, runtime.configuration),
               Filters::ApplyAroundHooks.new(FailingAroundHookAfterRunningTestCase.new),
               AddBeforeAndAfterHooks.new
             ]
@@ -2004,6 +2071,28 @@ module Cucumber
         end
       end
 
+      describe 'before_step_result message' do
+        context 'when the step matches' do
+          it "sends the step match to the formatter" do
+            expect(formatter).to receive(:before_step_result) do |_, step_match, *|
+              expect(step_match).to be_a SimpleStepMatch
+            end
+            execute_gherkin do
+              feature do
+                scenario do
+                  step 'passing'
+                end
+              end
+            end
+          end
+        end
+
+        context "when the step doesn't match" do
+          it "sends a null object to the formatter" do
+          end
+        end
+      end
+
       it 'passes nil as the multiline arg when there is none' do
         expect(formatter).to receive(:after_step_result) do |keyword, step_match, multiline_arg, status, exception, source_indent, background, file_colon_line|
           expect(multiline_arg).to be_nil
@@ -2017,14 +2106,17 @@ module Cucumber
         end
       end
 
-      it 'passes an object responding to failed? with the after_feature_element message' do
-        expect( formatter ).to receive(:after_feature_element) do |scenario|
-          expect( scenario ).to be_failed
-        end
-        execute_gherkin do
-          feature do
-            scenario do
-              step 'failing'
+      context "after_feature_element callback" do
+        it 'passes an object reflecting the status of the scenario' do
+          expect( formatter ).to receive(:after_feature_element).once do |scenario|
+            expect( scenario ).to be_failed
+          end
+          execute_gherkin do
+            feature do
+              scenario do
+                step 'failing'
+                step 'this will be skipped'
+              end
             end
           end
         end
@@ -2080,7 +2172,7 @@ module Cucumber
 
       def default_filters
         [
-          Filters::ActivateSteps.new(SimpleStepDefinitions.new),
+          Filters::ActivateSteps.new(step_match_search, runtime.configuration),
           AddBeforeAndAfterHooks.new
         ]
       end
