@@ -39,6 +39,21 @@ module Cucumber
         expect( @table.rows.first ).to eq %w{4444 55555 666666}
       end
 
+      describe '#symbolic_hashes' do
+
+        it 'should covert data table to an array of hashes with symbols as keys' do
+          ast_table = Cucumber::Core::Ast::DataTable.new([['foo', 'Bar', 'Foo Bar'], %w{1 22 333}], nil)
+          data_table = DataTable.new(ast_table)
+          expect(data_table.symbolic_hashes).to eq([{:foo => '1', :bar => '22', :foo_bar => '333'}])
+        end
+
+        it 'should be able to be accessed multiple times' do
+          @table.symbolic_hashes
+          expect{@table.symbolic_hashes}.to_not raise_error
+        end
+
+      end
+
       describe '#map_column!' do
         it "should allow mapping columns" do
           @table.map_column!('one') { |v| v.to_i }
@@ -316,6 +331,80 @@ module Cucumber
           }
         end
 
+        context "in case of duplicate header values" do
+          it "raises no error for two identical tables" do
+            t = DataTable.from(%{
+            |a|a|c|
+            |d|e|f|
+            |g|h|i|
+                               })
+            t.diff!(t.dup)
+            expect( t.to_s(:indent => 12, :color => false) ).to eq %{
+            |     a |     a |     c |
+            |     d |     e |     f |
+            |     g |     h |     i |
+          }
+          end
+
+          it "detects a diff in one cell" do
+            t1 = DataTable.from(%{
+            |a|a|c|
+            |d|e|f|
+            |g|h|i|
+                                })
+            t2 = DataTable.from(%{
+            |a|a|c|
+            |d|oops|f|
+            |g|h|i|
+                                })
+            expect{ t1.diff!(t2) }.to raise_error
+            expect( t1.to_s(:indent => 12, :color => false) ).to eq %{
+            |     a |     a    |     c |
+            | (-) d | (-) e    | (-) f |
+            | (+) d | (+) oops | (+) f |
+            |     g |     h    |     i |
+          }
+          end
+
+          it "detects missing columns" do
+            t1 = DataTable.from(%{
+            |a|a|b|c|
+            |d|d|e|f|
+            |g|g|h|i|
+                                })
+            t2 = DataTable.from(%{
+            |a|b|c|
+            |d|e|f|
+            |g|h|i|
+                                })
+            expect{ t1.diff!(t2) }.to raise_error
+            expect( t1.to_s(:indent => 12, :color => false) ).to eq %{
+            |     a | (-) a |     b |     c |
+            |     d | (-) d |     e |     f |
+            |     g | (-) g |     h |     i |
+          }
+          end
+
+          it "detects surplus columns" do
+            t1 = DataTable.from(%{
+            |a|b|c|
+            |d|e|f|
+            |g|h|i|
+                                })
+            t2 = DataTable.from(%{
+            |a|b|a|c|
+            |d|e|d|f|
+            |g|h|g|i|
+                                })
+            expect{ t1.diff!(t2, :surplus_col => true) }.to raise_error
+            expect( t1.to_s(:indent => 12, :color => false) ).to eq %{
+            |     a |     b |     c | (+) a |
+            |     d |     e |     f | (+) d |
+            |     g |     h |     i | (+) g |
+          }
+          end
+        end
+
         it "should inspect missing and surplus cells" do
           t1 = DataTable.from([
             ['name',  'male', 'lastname', 'swedish'],
